package com.sun.jmx.snmp.agent;

import java.util.ArrayList;
import javax.management.MBeanNotificationInfo;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpOid;
import java.util.List;
import java.io.Serializable;

public abstract class SnmpTableSupport implements SnmpTableEntryFactory, SnmpTableCallbackHandler, Serializable
{
    protected List<Object> entries;
    protected SnmpMibTable meta;
    protected SnmpMib theMib;
    private boolean registrationRequired;
    
    protected SnmpTableSupport(SnmpMib theMib) {
        this.registrationRequired = false;
        this.theMib = theMib;
        this.meta = this.getRegisteredTableMeta(theMib);
        this.bindWithTableMeta();
        this.entries = this.allocateTable();
    }
    
    @Override
    public abstract void createNewEntry(SnmpMibSubRequest p0, SnmpOid p1, int p2, SnmpMibTable p3) throws SnmpStatusException;
    
    public Object getEntry(int n) {
        if (this.entries == null) {
            return null;
        }
        return this.entries.get(n);
    }
    
    public int getSize() {
        return this.meta.getSize();
    }
    
    public void setCreationEnabled(boolean creationEnabled) {
        this.meta.setCreationEnabled(creationEnabled);
    }
    
    public boolean isCreationEnabled() {
        return this.meta.isCreationEnabled();
    }
    
    public boolean isRegistrationRequired() {
        return this.registrationRequired;
    }
    
    public SnmpIndex buildSnmpIndex(SnmpOid snmpOid) throws SnmpStatusException {
        return this.buildSnmpIndex(snmpOid.longValue(false), 0);
    }
    
    public abstract SnmpOid buildOidFromIndex(SnmpIndex p0) throws SnmpStatusException;
    
    public abstract ObjectName buildNameFromIndex(SnmpIndex p0) throws SnmpStatusException;
    
    @Override
    public void addEntryCb(int n, SnmpOid snmpOid, ObjectName objectName, Object o, SnmpMibTable snmpMibTable) throws SnmpStatusException {
        try {
            if (this.entries != null) {
                this.entries.add(n, o);
            }
        }
        catch (Exception ex) {
            throw new SnmpStatusException(2);
        }
    }
    
    @Override
    public void removeEntryCb(int n, SnmpOid snmpOid, ObjectName objectName, Object o, SnmpMibTable snmpMibTable) throws SnmpStatusException {
        try {
            if (this.entries != null) {
                this.entries.remove(n);
            }
        }
        catch (Exception ex) {}
    }
    
    public void addNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) {
        this.meta.addNotificationListener(notificationListener, notificationFilter, o);
    }
    
    public synchronized void removeNotificationListener(NotificationListener notificationListener) throws ListenerNotFoundException {
        this.meta.removeNotificationListener(notificationListener);
    }
    
    public MBeanNotificationInfo[] getNotificationInfo() {
        return this.meta.getNotificationInfo();
    }
    
    protected abstract SnmpIndex buildSnmpIndex(long[] p0, int p1) throws SnmpStatusException;
    
    protected abstract SnmpMibTable getRegisteredTableMeta(SnmpMib p0);
    
    protected List<Object> allocateTable() {
        return new ArrayList();
    }
    
    protected void addEntry(SnmpIndex snmpIndex, Object o) throws SnmpStatusException {
        SnmpOid buildOidFromIndex = this.buildOidFromIndex(snmpIndex);
        ObjectName buildNameFromIndex = null;
        if (this.isRegistrationRequired()) {
            buildNameFromIndex = this.buildNameFromIndex(snmpIndex);
        }
        this.meta.addEntry(buildOidFromIndex, buildNameFromIndex, o);
    }
    
    protected void addEntry(SnmpIndex snmpIndex, ObjectName objectName, Object o) throws SnmpStatusException {
        this.meta.addEntry(this.buildOidFromIndex(snmpIndex), objectName, o);
    }
    
    protected void removeEntry(SnmpIndex snmpIndex, Object o) throws SnmpStatusException {
        this.meta.removeEntry(this.buildOidFromIndex(snmpIndex), o);
    }
    
    protected Object[] getBasicEntries() {
        if (this.entries == null) {
            return null;
        }
        Object[] array = new Object[this.entries.size()];
        this.entries.toArray(array);
        return array;
    }
    
    protected void bindWithTableMeta() {
        if (this.meta == null) {
            return;
        }
        this.registrationRequired = this.meta.isRegistrationRequired();
        this.meta.registerEntryFactory(this);
        return;
    }
}
